from PyDictAPI import Finder, Translate
import requests
F = Finder(jsonify=True)
T = Translate()


#print(T.languages_help())
#print(T.translateItems("help", "hi"))
'''
print(F.findMeanings("help"))

print(F.findSynonyms("help"))
print(F.findAntonyms("help"))
print(F.findUsage("help"))
'''

input()


"""
Author: Shawan Mandal
    
MIT License, see LICENSE for more details.
Copyright (c) 2021 Shawan Mandal

"""

import sys, json
try:
    from utils import handleRequests, handleCorrection, getSoupObj, ParseUsage, ParseSynonymsAndAntonyms
except:
    from .utils import handleRequests, handleCorrection, getSoupObj, ParseUsage,  ParseSynonymsAndAntonyms

class PythonVersionError(Exception):
    pass

class Finder(object):
    """
    Finder
    Usage:
        >>> Meanings = Finder()
        >>> print(Meanings.findMeanings('apple'))
        >>> print(Meanings.findUsage('apple', 8))
        >>> print(Meanings.findSynonyms('apple', 8))
        >>> print(meanings.findAntonyms('apple', 12))
    """
    def __init__(self, jsonify=False):
        self.__jsonify=jsonify
        self.searching = "Please wait while I'm searching for "
        self.isPython3 = True
        if (sys.version_info.major) < 3:
            self.isPython3 = False
        else:
            pass

    def __IfnotFound(self, query):
        '''
        1.  Returns any possible matches incase if the queried word is not found
        2.  Returns a resolution incase if nothing is found
        '''
        errorString = {"message": f"Couldn't find any results for {query.upper()}, try searching the web..."}

        res = handleCorrection(query)
        soup = getSoupObj(res)

        try:
            suggestedContent = soup.find('h2', attrs={'class': 'spell-suggestions-subtitle'})
            suggestedWord = suggestedContent.find('a')
            resolution = {"message": f"Couldn't find results for {query}, Did you mean {suggestedWord.text}?"}
        except:
            resolution = errorString

        if self.__jsonify:
            return json.dumps(resolution, indent=2, ensure_ascii=False)
        else:
            return resolution["message"]
    
    def findMeanings(self, query):
        '''
        Searches for a word and returns response in a python Dictionary Obj,
        Alternatively searches for any possible matches incase the queried word is not found
        '''
        if (self.isPython3):
            pass
        else:
            raise PythonVersionError("Python version 3 or newer is required")

        print(self.searching + "meanings...")
        res = handleRequests(query)
        soup = getSoupObj(res)
        _pOs = None
        partOfSpeech = ""
        dataItems = {
            "word": query.title(),
            "meanings": []
        }

        contents = soup.findAll('section', attrs={'class': 'e1hk9ate4'})
        for each in contents:
            defFound = True
            json_contents = {}
            partOfSpeech = each.find(attrs={'class': 'e1hk9ate3'})
            for pos in partOfSpeech.find_all("span", {'class':'luna-inflected-form'}):
                    pos.replaceWith('')
            for pos1 in partOfSpeech.find_all("span", {'class':'inflected-form'}):
                    pos1.replaceWith('')
            for pos2 in partOfSpeech.find_all("span", {'class':'luna-pronset'}):
                    pos2.replaceWith('')
            partOfSpeech = partOfSpeech.get_text().title().strip()
            if partOfSpeech[-1] == ",":
                partOfSpeech = partOfSpeech[0:][:-1]
            json_contents = {
                "partOfSpeech": partOfSpeech,
                "definition": ""
            }
            definitions = each.findAll('div', attrs={'class': 'e1hk9ate0'})
            def_list = ""
            for definition in definitions:
                def_content = definition.find(attrs={'class': 'e1q3nk1v2'})
            
                if def_content:
                    for tag in def_content.find_all("span", {'class':'luna-example'}):
                        tag.replaceWith('')
                    def_content = def_content.get_text().replace('(', '').replace(')', '').replace(':', '').strip()
                    def_content = def_content[0].upper() + def_content[1:]
                else:
                    def_content = ''

                def_list += def_content
                def_list = def_list.strip()
            if def_list == "":
                defFound = False
            if defFound:
                json_contents['definition'] = def_list
                dataItems['meanings'].append(json_contents)
            else:
                pass
        if dataItems['meanings']:
            processedQuery = json.dumps(dataItems, indent=2, ensure_ascii=False)
        else:
            processedQuery = self.__IfnotFound(query)
        processedData = []
        if self.__jsonify:
            return processedQuery
        else:
            if dataItems['meanings']:
                for items in dataItems['meanings']:
                    data = []
                    data.append(items['partOfSpeech'])
                    data.append(items['definition'])
                    processedData.append(data)
                return processedData

        #return word, dataItems

    def findUsage(self, query, maxItems=5):
        """
        getUsage
        -----
        Returns a Python Dictionary of usage examples \n
        Args: Query -> (string), Maximum items -> (int) By default its value is 5

        Returns: \n
        {
            "word": [ ]
        }
        """
        if (self.isPython3):
            pass
        else:
            raise PythonVersionError("Python version 3 or newer is required")

        print(self.searching + "usage examples...")

        res = ParseUsage(query)
        soup = getSoupObj(res)
        usageExamples = {}
        examples = []
        try:
            usageClass = soup.find(attrs={'class': 'examples'})
            for junk in usageClass.find_all("div", {'class':'moreInfo active'}):
                            junk.replaceWith('')
            
            exg = usageClass.find(attrs={'class': 'exg'})
            ul = exg.find('ul').find_all('li')
            count = 0
            for each in ul:
                if count < maxItems:
                    text = each.text[1:][:-1]
                    examples.append(text[0].upper() + text[1:].strip())
                count += 1
        except:
            suggestions = self.__IfnotFound(query)
            examples.append(suggestions)
        usageExamples = {
            f"{query}": examples
        }
        if self.__jsonify:
            return json.dumps(usageExamples, indent=2, ensure_ascii=False)
        else:
            return examples
    
    def findSynonyms(self, query, maxItems=5):
        """
        findSynonyms
        ------------
        Returns a Python Dictionary of Synonyms \n
        Args: Query -> (string), Maximum items -> (int) By default its value is 5

        Returns: \n
        {
            "word": [ ]
        }
        """

        if (self.isPython3):
            pass
        else:
            raise PythonVersionError("Python version 3 or newer is required")

        print(self.searching + "Synonyms...")

        res = ParseSynonymsAndAntonyms(query)
        soup = getSoupObj(res)
        Syns = []
        Synonyms = {}

        try:
            count = 0
            SynonymsClass = soup.find(attrs={'class': 'e1ccqdb60'}).find_all('li')
            for each in SynonymsClass:
                if count < maxItems:
                    Syns.append(each.text[0].upper() + each.text[1:].strip())
                count += 1
        except:
            suggestions = self.__IfnotFound(query)
            Syns.append(suggestions)

        Synonyms = {
            f"{query}": Syns
        }
        if self.__jsonify:
            return json.dumps(Synonyms, indent=2, ensure_ascii=False)
        else:
            return Syns
        
    def findAntonyms(self, query, maxItems=5):
        """
        findAntonyms
        ------------
        Returns a Python Dictionary of Antonyms \n
        Args: Query -> (string), Maximum items -> (int) By default its value is 5

        Returns: \n
        {
            "word": [ ]
        }
        """

        if (self.isPython3):
            pass
        else:
            raise PythonVersionError("Python version 3 or newer is required")

        print(self.searching + "Antonyms...")

        res = ParseSynonymsAndAntonyms(query)
        soup = getSoupObj(res)
        Antyns = []
        Antonyms = {}
        count = 0

        try:
            AntonymsClass = soup.find(attrs={'id': 'antonyms'})
            Antonym = AntonymsClass.find(attrs={'class': 'css-ixatld e15rdun50'}).find('ul').find_all('li')
            for each in Antonym:
                if count < maxItems:
                    Antyns.append(each.text[0].upper() + each.text[1:].strip())
                count += 1
        except:
            suggestions = self.__IfnotFound(query)
            Antyns.append(suggestions)

        Antonyms = {
            f"{query}": Antyns
        }
        if self.__jsonify:
            return json.dumps(Antonyms, indent=2, ensure_ascii=False)
        else:
            return Antyns



"""
----------------------
Python Dictionary API 
----------------------

PyDictAPI is library written in Python, that can be used to fetch meanings and translation.

Both the Finder and Translator class takes an arguement "jsonify" that is set to False by default. 
If jsonify is set to True, than the processed queries are returned in JSON. While by default the queries are returned in the form of a Python List (Array)

Currently supports only English-English dictionary searches

Basic usage:

   >>> from PyDictAPI import Finder
   >>> Meanings = Finder(jsonify=True)
   >>> print(Meanings.findMeanings('apple'))

Output:

`{
  "word": "Apple",
  "meanings": [
    {
      "partOfSpeech": "Noun",
      "definition": "The usually round, red or yellow, edible fruit of a small tree, Malus sylvestris, of the rose family."  
    },
    {
      "partOfSpeech": "Noun",
      "definition": "A rosaceous tree, Malus sieversii, native to Central Asia but widely cultivated in temperate regions in many varieties, having pink or white fragrant flowers and firm rounded edible fruits. See also crab apple"
    }
  ]
}`

---------------------------------------    
Finding Examples, Synonyms and Antonyms
---------------------------------------

   >>> print(Meanings.findUsage('help', 2)) #Finding Examples
   
    # Here 2 defines the maximum number of examples to be included in the response, 
    # by default it is set to 5

   >>> print(Meanings.findSynonyms('help', 4)) #Finding Synonyms
   >>> print(Meanings.findAntonyms('help', 4)) #Finding Antonyms

----------------
Translating text
----------------

Example:

  >>> # Import the module first
  >>> from PyDictAPI import Translate
  >>> t = Translate(jsonify=True) #   Creates an instance of Translate class
  >>> 
  >>> # You can get all supported language list through languages_help()
  >>> languages = t.languages_help(pretty=True)
  >>> # Pretty=true returns the list of supported languages in a well structured manner. By default Pretty is set to False
  >>> 
  >>> # Tranlate English into Hindi
  >>> print(t.translateItems("Hello, How are you?", "hi"))

`{'query': 'Hello, How are you?', 'language_detected': 'Hindi', 'translation': 'नमस्कार किसे हो आप?'}`

Full documentation is at <https://github.com/imshawan/PyDictAPI>.

copyright: (c) 2021 by Shawan Mandal.

license: MIT License, see LICENSE for more details.
"""
__author__ = "Shawan Mandal"
__email__ = "imshawan.dev049@gmail.com"
__version__ = "1.6.0"

try:
    from .scrape import *
    from .translator import *
except:
    from scrape import *
    from translator import *





"""
Author: Shawan Mandal
    
MIT License, see LICENSE for more details.
Copyright (c) 2021 Shawan Mandal

"""

import urllib.request
import urllib.parse, sys, json, requests
from urllib.parse import urlencode

try:
    from utils import getSoupObj, handleRequests
except:
    from .utils import getSoupObj, handleRequests

class PythonVersionError(Exception):
    pass

class Translate(object):
    """
    ## PyDictAPI: Translator - The Free Translation API
    
    You have to first create an instance of Translate to use this API

    ### Example:
    >>> # Import the module first
    >>> from PyDictAPI import Translate
    >>> t = Translate(jsonify=True) #   Creates an instance of Translate class
    >>> # jsonify=True returns response in JSON, and by default jsonify is False
    >>> # You can get all supported language list through languages_help()
    >>> languages = t.languages_help())
    >>> 
    >>> # Tranlate English into Hindi
    >>> print(t.translateItems("Hello, How are you?", "hi"))

    `{'query': 'Hello, How are you?', 'language_detected': 'Hindi', 'translation': 'नमस्कार किसे हो आप?'}`
    """
    def __init__(self, jsonify=False):
        self.__jsonify = jsonify
        self.__searching = "Please wait while I process your query... \n"
        self.__CONTENT_HEADERS = {'User-Agent': 
        "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36"}
        self.isPython3 = True
        self.__SUPPORTED_LANGUAGES = {}
        self.__prettyText = "List of the Languages Supported... \n\n"
        
        if (sys.version_info.major) < 3:
            self.isPython3 = False
        else:
            pass
    
    def __prettyPrint(self):
        """
        ### Pretty prints text
        """
        SUPPORTED_LANGUAGES = json.loads(self.__SUPPORTED_LANGUAGES)
        data = SUPPORTED_LANGUAGES['sl']
        prettyText = self.__prettyText
        for key in data.keys():
            prettyText += key + ": \t" + data[key] + "\n"
        return prettyText

    def languages_help(self):
        '''
        # Returns supported languages

        It returns language codes for supported languages for translation. 
        Some language codes also include a country code, like zh-CN or zh-TW.

        :Example:

        >>> from PyDictAPI import Translate
        >>> t = Translate(jsonify=True)
        >>> print(t.languages_help())

        jsonify=True returns a json response,
        and by default jsonify is False

        '''

        if (self.isPython3):
            pass
        else:
            raise PythonVersionError("Python version 3 or newer is required")
        print(self.__searching)    
        if self.__SUPPORTED_LANGUAGES and self.__jsonify == False:
            return self.__SUPPORTED_LANGUAGES
        
        TRASLATOR_URL = 'http://translate.google.com/translate_a/l'
        TRASLATOR_PARAMS = {
            'client': 't',
            }
        url = '?'.join((TRASLATOR_URL, urlencode(TRASLATOR_PARAMS)))
        response_content = requests.get(url, headers=self.__CONTENT_HEADERS).text
        self.__SUPPORTED_LANGUAGES = json.loads(response_content)
        self.__SUPPORTED_LANGUAGES = json.dumps(self.__SUPPORTED_LANGUAGES, indent=2)
        if not self.__jsonify:
            return self.__prettyPrint()

        return self.__SUPPORTED_LANGUAGES

    def translateItems(self, query, translateLang="auto", from_lang="auto"):
        """
        Translates a word or sentence using google translate and returns the translated result.

        ### Example:
        >>> # Import the module first
        >>> from PyDictAPI import Translate
        >>> t = Translate(jsonify=true) #   Creates an instance of Translate class
        >>> 
        >>> # Tranlate English into Hindi
        >>> print(t.translateItems("Hello, How are you?", "hi"))

        `{'query': 'Hello, How are you?', 'language_detected': 'Hindi', 'translation': 'नमस्कार किसे हो आप?'}`
        """

        if (self.isPython3):
            pass
        else:
            raise PythonVersionError("Python version 3 or newer is required")
        text = query
        print(self.__searching)    

        query = urllib.parse.quote(query)
        URL = f"http://translate.google.com/m?tl={translateLang}&sl={from_lang}&q={query}"

        request = urllib.request.Request(URL, headers=self.__CONTENT_HEADERS)
        responseData = urllib.request.urlopen(request).read()
        data = getSoupObj(responseData)
        translatedList = []
        processedItem = ""
        Translation = {}
        try:
            lang = data.find(attrs={"class": "languages-container"}).find_all("a")
            lang = lang[1].text
        except:
            lang = "null"

        try:
            temp = data.findAll(attrs={"class": "result-container"})
            if len(temp) > 1:
                for each in temp:
                    translatedList.append(each.text)
                Translation = {
                "query": text,
                "language": lang,
                "translation": translatedList
                }
                processedItem = translatedList

            else:
                Translation = {
                "query": text,
                "language": lang,
                "translation": temp[0].text
                }
                processedItem = temp[0].text
        except:
            Translation = {
                "query": text,
                "message": "Couldn't translate your query, please try searching the web..."
                }
        if self.__jsonify:
            return json.dumps(Translation, indent=2, ensure_ascii=False)
        else:
            return processedItem